#pragma once

#include <memory>
#include <agents.h>
#include "connect.h"

//note the implementation is entirely in a header file
//for demo purposes as is the global using directive
using namespace ::Concurrency;
using namespace ::Concurrency::samples;

//make_unique helper functions
template <typename Type>
std::unique_ptr<Type> make_unique()
{
    return unique_ptr<Type>(new Type())
}

template <typename Type, typename Arg1>
std::unique_ptr<Type> make_unique(Arg1 && arg)
{
    return std::unique_ptr<Type>(new Type(std::forward<Arg1>(arg)));
}

template <typename Type, typename Arg1, typename Arg2>
std::unique_ptr<Type> make_unique(Arg1 && arg, Arg2 && arg2)
{
    return std::unique_ptr<Type>(new Type(std::forward<Arg1>(arg),
                                          std::forward<Arg2>(arg2)));
}

template <typename Type, typename Arg1, typename Arg2, typename Arg3>
std::unique_ptr<Type> make_unique(Arg1 && arg, Arg2 && arg2, Arg3 && arg3)
{
    return std::unique_ptr<Type>(new Type(std::forward<Arg1>(arg),
                                          std::forward<Arg2>(arg2),
                                          std::forward<Arg3>(arg3)));
}

template <typename Type, typename Arg1, typename Arg2, typename Arg3, typename Arg4>
std::unique_ptr<Type> make_unique(Arg1 && arg, Arg2 && arg2, Arg3 && arg3, Arg4 && arg4)
{
    return std::unique_ptr<Type>(new Type(std::forward<Arg1>(arg),
                                          std::forward<Arg2>(arg2),
                                          std::forward<Arg3>(arg3),
                                          std::forward<Arg4>(arg4)));
}

//timecall uses GetTickCount, don't use this where the slow update frequency is
//significant.
template <typename Func>
int timecall(Func fn)
{
    int begin = GetTickCount();
    fn();
    int end = GetTickCount();
    return end - begin;
}

// a simple atomic counter class
class atomic_counter
{
public:
    atomic_counter(long value = 0) : m_value(0)
    {
    }
    long increment()
    {
        return InterlockedIncrement(&m_value);
    }
    long value()
    {
        return m_value;
    }
    long operator=(long newValue)
    {
        return InterlockedExchange(&m_value, newValue);
    }
private:
    volatile long m_value;
};

//a global counter
atomic_counter frameCount = 0;

//signal is an empty struct
struct signal
{

};

//we exit after 250 frames
bool checkforinput()
{
    return (frameCount.value() < 250);
}

//render increments the frame count
void render_app()
{
    frameCount.increment();
    printf("rendering\n");
}


void update_app_state()
{
  printf("updating\n");
}



