#include <windows.h>
#include <ppl.h>
#include <agents.h>
#include <array>

#include "Globals.h"
#include "connect.h"

#include "RenderAgent.h"
#include "UpdateAgent.h"
#include "InputAgent.h"
#include "RenderDataFlow.h"
#include "UpdateDataFlow.h"
#include "InputDataFlow.h"

//this is the sample serial application loop.
void serial_app_loop()
{
    while(checkforinput())
    {
        update_app_state();
        render_app();
    }
}

//control_flow_network instantiates three control flow based agents
//starts them and waits for them to complete
void control_flow_network()
{
    //declare the three agents (renderer, updater, controller)
    render_agent renderer;
    update_agent updater;
    input_agent  controller;

    //start them
    renderer.start();
    updater.start();
    controller.start();

    //connect the controller to the updater
    connect(controller, updater);

    //the updater to the renderer
    connect(updater, renderer);

    //and the renderer back to the controller
    connect(renderer, controller);
    connect(controller.shutdown_output_buffer(), renderer.shutdown_input_buffer());
    connect(controller.shutdown_output_buffer(), updater.shutdown_input_buffer());

    //now start the loop...
    asend(controller.input_buffer(), signal());

    //wait for the agents to finish
    agent::wait(&renderer);
    agent::wait(&updater);
    agent::wait(&controller);
}

//data_flow_network instantiates three data flow agents
//starts them and waits for them to complete
void data_flow_network()
{
    render_agent_dataflow renderer;
    update_agent_dataflow updater;
    input_agent_dataflow  controller;

    renderer.start();
    updater.start();
    controller.start();

    //connect the update and render loop
    connect(controller, updater);
    connect(updater,    renderer);
    connect(renderer,   controller);

    //connect the off button
    connect(controller.shutdown_output_buffer(), renderer.shutdown_input_buffer());
    connect(controller.shutdown_output_buffer(), updater.shutdown_input_buffer());

    //now start the loop...
    asend(controller.input_buffer(),signal());

    agent::wait(&renderer);
    agent::wait(&updater);
    agent::wait(&controller);;
}

int main()
{
    int serial   = 0;
    int control  = 0;
    int dataflow = 0;

    // a small serial app loop
    serial = timecall(serial_app_loop);

    //reset
    frameCount = 0;

    // a control flow based app loop
    control = timecall(control_flow_network);

    //reset
    frameCount = 0;

    // a data flow based app loop
    dataflow = timecall(data_flow_network);

    printf("serial: %d ms, control: %d ms, dataflow: %d ms\n", serial, control, dataflow);

    return 0;
}
